function  [BN_cycle,pi_star,shock_decom,FEVD,MVSTNR,taupi,FS_STNR] =...
    BN_Inflation_BFAVAR_relax_block_exogenous_MCMC(y,pi_levels,p,lambda,nForFac,varargin)
% Benjamin Wong
% Monash University
% October 2019
% Estimates the Bayesian International FAVAR with and without the error
% correction term and subsequent performs
% BN decomposition and associated analysis to obtain trend inflation and
% Inflation gap as per
% Kamber and Wong "Global Factors and Trend Inflation"
%
% The Bayesian VAR with block exogeniety is estimated using a Gibbs Sampler
% with a Minnesota type prior with a standard Normal-Wishart Prior Structure
%
% The model is estimated without a constant.
% All equations are demeaned except the change in inflation to impose
% random walk with (as oppossed to without) drift in trend inflation
%
%% INPUTS
%
%y                        Time series except for the inflation rates
%pi_levels                Level of the inflation rate. This needs to be 1
%                         obs more than y. If cointegrating option is
%                         chosen, need level of headline and core
%p                        lags
%lambda                   Shrinkage on hyper parameters
%nForFac                  Number of variables in foreign block (for imposing block exogeniety)
%varargin                 Enter 'DoDecomp' to do FEVD, or the historical
%                         decomposition
%
% Inflation will be the last equation. If EC term is used, headline and
% core will be the last two equations.
%
%% OUTPUTS
%BN_cycle                   The estimated BN cycle calculated for all
%                           variables
%pi_star                    Estimatd Trend inflation. 2 columns if
%                           cointegration chosen to just check
%shock_decom                Stores the historical shock decomposition with the
%                           cholesky structure
%FEVD                       Stores the Forecast error variance
%                           decomposition with the cholesky structure
%MVSTNR                     Estimated Signal-to-noise ratio of inflation
% Note: Because of block exogeniety, cholesky structure matters little as
% foreign and domestic shocks are aggregated.

%% Put some obvious error messages
if size(pi_levels,1) - size(y,1) ~= 1
    error('Check Inflation and PCA data. Inflation data should have one more observation')
end

if sum(strcmp(varargin,'ECTerm'))==0 && size(pi_levels,2)~=1
    error('Error Correction Term turned off, but Inflation data has two columns')
end

if sum(strcmp(varargin,'ECTerm'))>0 && size(pi_levels,2)~=2
    error('Error Correction Term turned on, but Inflation data does not have two columns')
end

%% Preliminaries
y = [y diff(pi_levels)];
if sum(strcmp(varargin,'ECTerm'))>0
    coin_vect = pi_levels(:,1)-pi_levels(:,2);
end

[T N] = size(y);

%demean time series, do not demean delta inflation
if sum(strcmp(varargin,'ECTerm'))>0
    y(:,1:N-2) = y(:,1:N-2) - repmat(mean(y(:,1:N-2)),T,1);
else    % Demean all except last one
    y(:,1:N-1) = y(:,1:N-1) - repmat(mean(y(:,1:N-1)),T,1);
end

%backcast data
y = [zeros(p,N);y];

K = p*N;        %number of parameters per equation (without cointegration term)
K_foreign_block = p*nForFac; % number of estimated parameters per equation in foreign block
A_post = zeros(K,N); % To store estimated VAR parameters (no cointegration term for now)

small_sig2 =  zeros(N,1);


%Get sigma to set prior based on an AR(4)
for i = 1:N
    [~,small_sig2(i,1),~,~,~] = olsvar(y(:,i),p,'No Constant');
end

Y = y(p+1:end,:);     %Cut Away first p lags (the backcasted stuff)
X = [];

for i = 1:p
    Z = y(p+1-i:end-i,:);
    X = [X Z];
end


%% Set up Prior


a_prior = zeros(K*N,1);

V_prior = [];
for jj = 1:N
    for ii = 1:p
        V_prior = [V_prior;repmat((lambda^2)*small_sig2(jj)/(ii^2),N,1)./small_sig2(1:N,1)];
    end
end


V_prior = diag(V_prior);


%% Create Regressors
Z = kron(eye(N),X);

%%

% ML estimators
A_OLS = (X'*X)\(X'*Y);
a_OLS = A_OLS(:); % This is the vector of parameters, i.e. it holds
% that a_OLS = vec(A_OLS)
ols_resid = Y - X*A_OLS;
SSE = ols_resid'*ols_resid;   % Sum of squared errors
SIGMA_OLS = SSE./(T);


% Hyperparameters on inv(SIGMA) ~ W(v_prior,inv(S_prior))
v_prior = N+1;
S_prior = (N+1)*diag(small_sig2);


%% Initialize Bayesian posterior parameters using OLS values
alpha = a_OLS;     % This is the single draw from the posterior of alpha
SIGMA_draw = SIGMA_OLS+diag(small_sig2); % This is the single draw from the posterior of SIGMA

%% Gibbs sampler

% Storage space for posterior draws

num_MCMC_posterior_taken = 0;


ALPHA_KEEP = zeros(K,N);
SIGMA_KEEP = zeros(N,N);

%% MCMC
for jj = 1:6000
    
    
    VARIANCE = kron(inv(SIGMA_draw),eye(T));
    V_post = inv(inv(V_prior) + Z'*VARIANCE*Z);
    a_posterior = V_post*(inv(V_prior)*a_prior + Z'*VARIANCE*Y(:));
    alpha = a_posterior + chol(V_post,'lower')*randn(size(Z,2),1); % Draw of alpha
        
    ALPHA_draw = reshape(alpha,N*p,N);
    
    % Posterior of SIGMA|ALPHA,Data ~ iW(inv(S_post),v_post)
    v_post = T + v_prior;
    S_post = S_prior + (Y - X*ALPHA_draw)'*(Y - X*ALPHA_draw);
    
    
    
    SIGMA_draw = inv(wishrnd(inv(S_post),v_post));% Draw SIGMA
    

    if jj >2000 && mod(jj,2) == 0
        num_MCMC_posterior_taken = num_MCMC_posterior_taken+1;
        ALPHA_KEEP = ALPHA_KEEP + ALPHA_draw;
        SIGMA_KEEP = SIGMA_KEEP + SIGMA_draw;
    end
    
end

A_post = ALPHA_KEEP/num_MCMC_posterior_taken;
SIGMA = SIGMA_KEEP/num_MCMC_posterior_taken;


U = Y-X*A_post;


%% BN Decomposition Starts here
%Add current period in to incoporate current information set to do BN decomposition


X = [X(2:end,:);
    Y(end,:) X(end,1:end-N)];
I_block = [eye(N*(p-1)) zeros(N*(p-1),N)];
bigeye = eye(N*p);



Companion = [A_post';
    I_block];

phi = -Companion*((bigeye-Companion)\eye(N*p));


BN_cycle = phi*X';
BN_cycle = BN_cycle(1:N,:)';


pi_star = pi_levels(2:end,:) - BN_cycle(:,end);


%% Calculate Signal to Noise ratio of Inflation

RF_shock = zeros(K,1); RF_shock(N) = 1;
MVSTNR = (inv(eye(K)-Companion)*RF_shock).^2;
MVSTNR = MVSTNR(N,1);


%% Calculate the variance of trend inflation
% See Equation 9 and square equation 9 of Morley & Wong (2019)

I_minusF_inv = inv(eye(K)-Companion);
I_minusF_inv = I_minusF_inv(1:N,1:N);
taupi = I_minusF_inv*SIGMA*I_minusF_inv';
taupi = taupi(N,N);

%% Do information decompositon of the Cycle

shock_decom = [];
FEVD = [];


if sum(strcmp(varargin,'DoDecomp'))>0
    
    %construct historical structural shocks
    
    eta = zeros(size(U));
    A0 = chol(SIGMA,'lower');
    A0_inv = A0\eye(N);
    
    for ii = 1:T
        eta(ii,:) = A0_inv*U(ii,:)';
    end
    
    
    bigeye_selector_vec = [eye(N) zeros(N,N*(p-1))];
    shock_decom.cycle = zeros(N,N,size(X,1));
    shock_decom.trend = zeros(N,N,size(X,1));
    
    
    % see formulaes in the paper
    for jj = 1:size(X,1)
        count = 0;
        
        for ii = jj:-1:1
            shock_vec = diag(U(ii,:));
            
            s_shocks_vec = diag(eta(ii,:));
            shock_temp = A0*s_shocks_vec;
            
            
            if ii == jj
                shock_decom.trend(:,:,jj) = bigeye_selector_vec*((bigeye-Companion)\eye(size(Companion,1)))*bigeye_selector_vec'*shock_temp;
            end
            
            shock_decom.cycle(:,:,jj)= shock_decom.cycle(:,:,jj)+ ...
                bigeye_selector_vec*(phi*(Companion^count))*bigeye_selector_vec'*shock_temp;
            
            count = count + 1;
        end
    end
    %% Calculate FEVD
    
    
    A0_prime = A0;
    
    
    % See Kamber, Morley and Wong (REStat, 2018, online appendix, Section A4)
    inv_I_minus_F_Kon_F = (eye(size(Companion,1)^2)-kron(Companion,Companion))\eye(size(Companion,1)^2);
    
    for jj = 1:N
        BigSIGMA = zeros(size(Companion));
        
        BigSIGMA(1:N,1:N) = A0_prime(:,jj)*A0_prime(:,jj)';
        
        SIGMA_X = inv_I_minus_F_Kon_F*BigSIGMA(:);
        SIGMA_X = reshape(SIGMA_X,size(Companion,1),size(Companion,1));
        cycle_conditional_variance_temp = phi*SIGMA_X*phi';
        cycle_conditional_variance(:,jj) = diag(cycle_conditional_variance_temp(1:N,1:N));
    end
    
    FEVD.cycle = 100*cycle_conditional_variance./repmat(sum(cycle_conditional_variance,2),1,N);
    
    %trend
    
    BigPhi = (bigeye-Companion)\eye(size(Companion,2));
    
    FEVD.trend = reshape(bigeye_selector_vec*BigPhi*bigeye_selector_vec'*A0_prime,[],N);
    FEVD.trend = FEVD.trend.^2;
    
    FEVD.trend = 100*(FEVD.trend./repmat(sum(FEVD.trend,2),1,N));
    
end

%% Calculate Foreign Shocks STNR
%Calculate the variance of change in inflation due to foreign shocks
variance_pi_f = sum(A0(end,1:nForFac).^2);

%Calculates the variance of trend inflation (i.e. taupi) due to trend
%inflation

variance_tau_f = sum(FEVD.trend(end,1:nForFac))*taupi/100;


FS_STNR = variance_tau_f/variance_pi_f;

end